@@ -35,6 +35,7 @@
 #include "client.h"
 
 gboolean                 compositorIsUsable                     (DisplayInfo *);
+gboolean                 compositorIsActive                     (ScreenInfo *);
 void                     compositorAddWindow                    (DisplayInfo *,
                                                                  Window,
                                                                  Client *);
@@ -51,6 +52,10 @@ void                     compositorResizeWindow                 (DisplayInfo *,
                                                                  int,
                                                                  int,
                                                                  int);
+Pixmap                   compositorGetWindowPixmapAtSize        (ScreenInfo *,
+                                                                 Window,
+                                                                 guint *,
+                                                                 guint *);
 void                     compositorHandleEvent                  (DisplayInfo *,
                                                                  XEvent *);
 void                     compositorZoomIn                       (ScreenInfo *,
@@ -70,7 +75,7 @@ void                     compositorUpdateScreenSize             (ScreenInfo *);
 
 void                     compositorWindowSetOpacity             (DisplayInfo *,
                                                                  Window,
-                                                                 guint);
+                                                                 guint32);
 void                     compositorRebuildScreen                (ScreenInfo *);
 gboolean                 compositorTestServer                   (DisplayInfo *);
 
