@@ -852,16 +852,21 @@ titleButton (Client * c, guint state, XButtonEvent * ev)
             }
         }
     }
-    else if (ev->button == Button6)
+#ifdef HAVE_COMPOSITOR
+    else if (screen_info->params->horiz_scroll_opacity)
     {
-        /* Mouse wheel scroll left, or left side button */
-        clientDecOpacity(c);
-    }
-    else if (ev->button == Button7)
-    {
-        /* Mouse wheel scroll right, or right side button */
-        clientIncOpacity(c);
+        if (ev->button == Button6)
+        {
+            /* Mouse wheel scroll left, or left side button */
+            clientDecOpacity(c);
+        }
+        else if (ev->button == Button7)
+        {
+            /* Mouse wheel scroll right, or right side button */
+            clientIncOpacity(c);
+        }
     }
+#endif /* HAVE_COMPOSITOR */
 }
 
 static void
@@ -1357,8 +1362,8 @@ handleConfigureRequest (DisplayInfo *display_info, XConfigureRequestEvent * ev)
             /* Sorry, but it's not the right time for configure request */
             return EVENT_FILTER_REMOVE;
         }
+        clientAdjustCoordGravity (c, c->gravity, &wc, &ev->value_mask);
         clientMoveResizeWindow (c, &wc, ev->value_mask);
-        clientAdjustCoordGravity (c, c->gravity, &ev->value_mask, &wc);
     }
     else
     {
