@@ -44,6 +44,14 @@ int                      frameX                                 (Client *);
 int                      frameY                                 (Client *);
 int                      frameWidth                             (Client *);
 int                      frameHeight                            (Client *);
+int                      frameExtentLeft                        (Client *);
+int                      frameExtentRight                       (Client *);
+int                      frameExtentTop                         (Client *);
+int                      frameExtentBottom                      (Client *);
+int                      frameExtentX                           (Client *);
+int                      frameExtentY                           (Client *);
+int                      frameExtentWidth                       (Client *);
+int                      frameExtentHeight                      (Client *);
 void                     frameSetShapeInput                     (Client *);
 void                     frameClearQueueDraw                    (Client *);
 void                     frameQueueDraw                         (Client *,
