@@ -18,7 +18,7 @@
 
         Metacity - (c) 2001 Havoc Pennington
         libwnck  - (c) 2001 Havoc Pennington
-        xfwm4    - (c) 2002-2011 Olivier Fourdan
+        xfwm4    - (c) 2002-2015 Olivier Fourdan
  */
 
 #ifdef HAVE_CONFIG_H
@@ -36,15 +36,41 @@
 #include "inline-default-icon.h"
 #include "icons.h"
 #include "display.h"
+#include "screen.h"
+#include "client.h"
+#include "compositor.h"
 #include "hints.h"
 
+static void
+downsize_ratio (guint *width, guint *height, guint dest_w, guint dest_h)
+{
+    gdouble ratio;
+    guint size;
+
+    g_return_if_fail (width != NULL);
+    g_return_if_fail (height != NULL);
+    g_return_if_fail (dest_w > 0 && dest_w > 0);
+
+    size = MIN (dest_w, dest_h);
+    if (*width > *height)
+    {
+        ratio = ((gdouble) *width) / size;
+        *width = (guint) size;
+        *height = (guint) (((gdouble) *height) / ratio);
+    }
+    else
+    {
+        ratio = ((gdouble) *height) / size;
+        *height = (guint) size;
+        *width = (guint) (((gdouble) *width) / ratio);
+    }
+}
+
 /*
  * create a GdkPixbuf from inline data and scale it to a given size
  */
 static GdkPixbuf *
-inline_icon_at_size (const guint8 *data,
-                          int width,
-                          int height)
+inline_icon_at_size (const guint8 *data, guint width, guint height)
 {
     GdkPixbuf *base;
 
@@ -52,23 +78,21 @@ inline_icon_at_size (const guint8 *data,
 
     g_return_val_if_fail (base, NULL);
 
-    if ((width < 0 && height < 0)
-        || (gdk_pixbuf_get_width (base) == width
-            && gdk_pixbuf_get_height (base) == height))
+    if ((width <= 0 || height <= 0) ||
+        ((guint) gdk_pixbuf_get_width (base) == width &&
+         (guint) gdk_pixbuf_get_height (base) == height))
     {
         return base;
     }
     else
     {
         GdkPixbuf *scaled;
+        guint w, h;
 
-        scaled = gdk_pixbuf_scale_simple (base,
-                                          width >
-                                          0 ? width : gdk_pixbuf_get_width (base),
-                                          height >
-                                          0 ? height :
-                                          gdk_pixbuf_get_height (base),
-                                          GDK_INTERP_BILINEAR);
+        w = gdk_pixbuf_get_width (base);
+        h = gdk_pixbuf_get_height (base);
+        downsize_ratio (&w, &h, width, height);
+        scaled = gdk_pixbuf_scale_simple (base, w, h, GDK_INTERP_BILINEAR);
 
         g_object_unref (G_OBJECT (base));
 
@@ -78,9 +102,9 @@ inline_icon_at_size (const guint8 *data,
 
 
 static gboolean
-find_largest_sizes (gulong * data, gulong nitems, int *width, int *height)
+find_largest_sizes (gulong * data, gulong nitems, guint *width, guint *height)
 {
-    int w, h;
+    guint w, h;
 
     *width = 0;
     *height = 0;
@@ -111,13 +135,13 @@ find_largest_sizes (gulong * data, gulong nitems, int *width, int *height)
 }
 
 static gboolean
-find_best_size (gulong * data, gulong nitems, int ideal_width, int ideal_height,
-                int *width, int *height, gulong ** start)
+find_best_size (gulong * data, gulong nitems, gint ideal_width, gint ideal_height,
+                guint *width, guint *height, gulong ** start)
 {
     gulong *best_start;
-    int ideal_size;
-    int w, h, best_size, this_size;
-    int best_w, best_h, max_width, max_height;
+    guint ideal_size;
+    guint w, h, best_size, this_size;
+    guint best_w, best_h, max_width, max_height;
 
     *width = 0;
     *height = 0;
@@ -211,12 +235,12 @@ find_best_size (gulong * data, gulong nitems, int ideal_width, int ideal_height,
 }
 
 static void
-argbdata_to_pixdata (gulong * argb_data, int len, guchar ** pixdata)
+argbdata_to_pixdata (gulong * argb_data, guint len, guchar ** pixdata)
 {
     guchar *p;
     guint argb;
     guint rgba;
-    int i;
+    guint i;
 
     *pixdata = g_new (guchar, len * 4);
     p = *pixdata;
@@ -237,13 +261,13 @@ argbdata_to_pixdata (gulong * argb_data, int len, guchar ** pixdata)
 }
 
 static gboolean
-read_rgb_icon (DisplayInfo *display_info, Window window, int ideal_width, int ideal_height,
-               int *width, int *height, guchar ** pixdata)
+read_rgb_icon (DisplayInfo *display_info, Window window, guint ideal_width, guint ideal_height,
+               guint *width, guint *height, guchar ** pixdata)
 {
     gulong nitems;
     gulong *data;
     gulong *best;
-    int w, h;
+    guint w, h;
 
     data = NULL;
 
@@ -269,7 +293,7 @@ read_rgb_icon (DisplayInfo *display_info, Window window, int ideal_width, int id
 }
 
 static void
-get_pixmap_geometry (Display *dpy, Pixmap pixmap, unsigned int *w, unsigned int *h)
+get_pixmap_geometry (Display *dpy, Pixmap pixmap, guint *w, guint *h)
 {
     Window root;
     guint border_width;
@@ -285,8 +309,8 @@ apply_mask (GdkPixbuf * pixbuf, GdkPixbuf * mask)
     GdkPixbuf *with_alpha;
     guchar *src;
     guchar *dest;
-    int w, h, i, j;
-    int src_stride, dest_stride;
+    guint w, h, i, j;
+    guint src_stride, dest_stride;
 
     w = MIN (gdk_pixbuf_get_width (mask), gdk_pixbuf_get_width (pixbuf));
     h = MIN (gdk_pixbuf_get_height (mask), gdk_pixbuf_get_height (pixbuf));
@@ -328,7 +352,7 @@ apply_mask (GdkPixbuf * pixbuf, GdkPixbuf * mask)
 }
 
 static GdkColormap *
-get_cmap (GdkPixmap * pixmap)
+get_cmap (GdkPixmap * pixmap, GdkScreen *gscreen)
 {
     GdkColormap *cmap;
 
@@ -337,6 +361,7 @@ get_cmap (GdkPixmap * pixmap)
     cmap = gdk_drawable_get_colormap (pixmap);
     if (cmap)
     {
+        g_message ("Drawable colormap");
         g_object_ref (G_OBJECT (cmap));
     }
     else
@@ -346,6 +371,12 @@ get_cmap (GdkPixmap * pixmap)
             /* try null cmap */
             cmap = NULL;
         }
+        else if ((gdk_drawable_get_depth (pixmap) == 32) && (gscreen != NULL))
+        {
+            /* Try ARGB cmap */
+            cmap = gdk_screen_get_rgba_colormap(gscreen);
+            g_object_ref (G_OBJECT (cmap));
+        }
         else
         {
             /* Try system cmap */
@@ -365,9 +396,8 @@ get_cmap (GdkPixmap * pixmap)
 }
 
 static GdkPixbuf *
-get_pixbuf_from_pixmap (GdkPixbuf * dest, Pixmap xpixmap,
-                        int src_x, int src_y, int dest_x,
-                        int dest_y, int width, int height)
+get_pixbuf_from_pixmap (GdkScreen *gscreen, Pixmap xpixmap, guint src_x, guint src_y,
+                        gint dest_x, gint dest_y, guint width, guint height)
 {
     GdkDrawable *drawable;
     GdkPixbuf *retval;
@@ -392,9 +422,8 @@ get_pixbuf_from_pixmap (GdkPixbuf * dest, Pixmap xpixmap,
         return NULL;
     }
 
-    cmap = get_cmap (drawable);
-
-    retval = gdk_pixbuf_get_from_drawable (dest, drawable, cmap, src_x, src_y,
+    cmap = get_cmap (drawable, gscreen);
+    retval = gdk_pixbuf_get_from_drawable (NULL, drawable, cmap, src_x, src_y,
                                            dest_x, dest_y, width, height);
 
     if (G_LIKELY(cmap))
@@ -407,12 +436,12 @@ get_pixbuf_from_pixmap (GdkPixbuf * dest, Pixmap xpixmap,
 }
 
 static GdkPixbuf *
-try_pixmap_and_mask (Display *dpy, Pixmap src_pixmap, Pixmap src_mask, int width, int height)
+try_pixmap_and_mask (ScreenInfo *screen_info, Pixmap src_pixmap, Pixmap src_mask, guint width, guint height)
 {
     GdkPixbuf *unscaled;
     GdkPixbuf *icon;
     GdkPixbuf *mask;
-    unsigned int w, h;
+    guint w, h;
 
     if (src_pixmap == None)
     {
@@ -420,15 +449,15 @@ try_pixmap_and_mask (Display *dpy, Pixmap src_pixmap, Pixmap src_mask, int width
     }
 
     gdk_error_trap_push ();
-    get_pixmap_geometry (dpy, src_pixmap, &w, &h);
-    unscaled = get_pixbuf_from_pixmap (NULL, src_pixmap, 0, 0, 0, 0, w, h);
+    get_pixmap_geometry (myScreenGetXDisplay(screen_info), src_pixmap, &w, &h);
+    unscaled = get_pixbuf_from_pixmap (screen_info->gscr, src_pixmap, 0, 0, 0, 0, w, h);
     icon = NULL;
     mask = NULL;
 
     if (unscaled && src_mask)
     {
-        get_pixmap_geometry (dpy, src_mask, &w, &h);
-        mask = get_pixbuf_from_pixmap (NULL, src_mask, 0, 0, 0, 0, w, h);
+        get_pixmap_geometry (myScreenGetXDisplay(screen_info), src_mask, &w, &h);
+        mask = get_pixbuf_from_pixmap (screen_info->gscr, src_mask, 0, 0, 0, 0, w, h);
     }
     gdk_error_trap_pop ();
 
@@ -446,7 +475,8 @@ try_pixmap_and_mask (Display *dpy, Pixmap src_pixmap, Pixmap src_mask, int width
 
     if (unscaled)
     {
-        icon = gdk_pixbuf_scale_simple (unscaled, width, height, GDK_INTERP_BILINEAR);
+        downsize_ratio (&w, &h, width, height);
+        icon = gdk_pixbuf_scale_simple (unscaled, w, h, GDK_INTERP_BILINEAR);
         g_object_unref (G_OBJECT (unscaled));
         return icon;
     }
@@ -461,12 +491,10 @@ free_pixels (guchar * pixels, gpointer data)
 }
 
 static GdkPixbuf *
-scaled_from_pixdata (guchar * pixdata, int w, int h, int new_w, int new_h)
+scaled_from_pixdata (guchar * pixdata, guint w, guint h, guint dest_w, guint dest_h)
 {
     GdkPixbuf *src;
     GdkPixbuf *dest;
-    GdkPixbuf *tmp;
-    int size;
 
     src = gdk_pixbuf_new_from_data (pixdata, GDK_COLORSPACE_RGB, TRUE, 8, w, h, w * 4, free_pixels, NULL);
 
@@ -475,25 +503,10 @@ scaled_from_pixdata (guchar * pixdata, int w, int h, int new_w, int new_h)
         return NULL;
     }
 
-    if (w != h)
+    if (w != dest_w || h != dest_h)
     {
-        size = MAX (w, h);
-
-        tmp = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, size, size);
-
-        if (G_LIKELY(tmp != NULL))
-        {
-            gdk_pixbuf_fill (tmp, 0);
-            gdk_pixbuf_copy_area (src, 0, 0, w, h, tmp, (size - w) / 2, (size - h) / 2);
-
-            g_object_unref (src);
-            src = tmp;
-        }
-    }
-
-    if (w != new_w || h != new_h)
-    {
-        dest = gdk_pixbuf_scale_simple (src, new_w, new_h, GDK_INTERP_BILINEAR);
+        downsize_ratio (&w, &h, dest_w, dest_h);
+        dest = gdk_pixbuf_scale_simple (src, w, h, GDK_INTERP_BILINEAR);
         g_object_unref (G_OBJECT (src));
     }
     else
@@ -505,25 +518,25 @@ scaled_from_pixdata (guchar * pixdata, int w, int h, int new_w, int new_h)
 }
 
 GdkPixbuf *
-getAppIcon (DisplayInfo *display_info, Window window, int width, int height)
+getAppIcon (ScreenInfo *screen_info, Window window, guint width, guint height)
 {
     XWMHints *hints;
     Pixmap pixmap;
     Pixmap mask;
     guchar *pixdata;
-    int w, h;
+    guint w, h;
 
     pixdata = NULL;
     pixmap = None;
     mask = None;
 
-    if (read_rgb_icon (display_info, window, width, height, &w, &h, &pixdata))
+    if (read_rgb_icon (screen_info->display_info, window, width, height, &w, &h, &pixdata))
     {
         return scaled_from_pixdata (pixdata, w, h, width, height);
     }
 
     gdk_error_trap_push ();
-    hints = XGetWMHints (display_info->dpy, window);
+    hints = XGetWMHints (myScreenGetXDisplay(screen_info), window);
     gdk_error_trap_pop ();
 
     if (hints)
@@ -543,17 +556,17 @@ getAppIcon (DisplayInfo *display_info, Window window, int width, int height)
 
     if (pixmap != None)
     {
-        GdkPixbuf *icon = try_pixmap_and_mask (display_info->dpy, pixmap, mask, width, height);
+        GdkPixbuf *icon = try_pixmap_and_mask (screen_info, pixmap, mask, width, height);
         if (icon)
         {
             return icon;
         }
     }
 
-    getKDEIcon (display_info, window, &pixmap, &mask);
+    getKDEIcon (screen_info->display_info, window, &pixmap, &mask);
     if (pixmap != None)
     {
-        GdkPixbuf *icon = try_pixmap_and_mask (display_info->dpy, pixmap, mask, width, height);
+        GdkPixbuf *icon = try_pixmap_and_mask (screen_info, pixmap, mask, width, height);
         if (icon)
         {
             return icon;
@@ -562,3 +575,69 @@ getAppIcon (DisplayInfo *display_info, Window window, int width, int height)
 
     return inline_icon_at_size (default_icon_data, width, height);
 }
+
+GdkPixbuf *
+getClientIcon (Client *c, guint width, guint height)
+{
+    ScreenInfo *screen_info;
+    GdkPixbuf *app_content;
+    GdkPixbuf *small_icon;
+    GdkPixbuf *icon_pixbuf;
+    GdkPixbuf *icon_pixbuf_stated;
+    guint small_icon_size;
+    guint app_icon_width, app_icon_height;
+    Pixmap pixmap;
+
+    g_return_val_if_fail (c != NULL, NULL);
+
+    screen_info = c->screen_info;
+    icon_pixbuf = NULL;
+    app_icon_width = width; /* Set to 0 to use gdk pixbuf scaling */
+    app_icon_height = height; /* Set to 0 to use gdk pixbuf scaling */
+
+    icon_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, width, height);
+    gdk_pixbuf_fill (icon_pixbuf, 0x00);
+
+    pixmap = compositorGetWindowPixmapAtSize (screen_info, c->frame, &app_icon_width, &app_icon_height);
+    if (pixmap != None)
+    {
+        app_content = try_pixmap_and_mask (screen_info, pixmap, None, width, height);
+        XFreePixmap (myScreenGetXDisplay (screen_info), pixmap);
+    }
+    else
+    {
+        app_content = inline_icon_at_size (default_icon_data, width, height);
+    }
+
+    app_icon_width = (guint) gdk_pixbuf_get_width (app_content);
+    app_icon_height = (guint) gdk_pixbuf_get_height (app_content);
+
+    gdk_pixbuf_copy_area (app_content, 0, 0, app_icon_width, app_icon_height, icon_pixbuf,
+                          (width - app_icon_width) / 2, (height - app_icon_height) / 2);
+    g_object_unref (app_content);
+
+    small_icon_size = MIN (width / 4, height / 4);
+    small_icon_size = MIN (small_icon_size, 48);
+
+    small_icon = getAppIcon (screen_info, c->window, small_icon_size, small_icon_size);
+
+    gdk_pixbuf_composite (small_icon, icon_pixbuf,
+                          (width - small_icon_size) / 2, height - small_icon_size,
+                          small_icon_size, small_icon_size,
+                          (width - small_icon_size) / 2, height - small_icon_size,
+                          1.0, 1.0,
+                          GDK_INTERP_BILINEAR,
+                          0xff);
+
+    g_object_unref (small_icon);
+
+    if (FLAG_TEST (c->flags, CLIENT_FLAG_ICONIFIED))
+    {
+        icon_pixbuf_stated = gdk_pixbuf_copy (icon_pixbuf);
+        gdk_pixbuf_saturate_and_pixelate (icon_pixbuf, icon_pixbuf_stated, 0.55, TRUE);
+        g_object_unref (icon_pixbuf);
+        icon_pixbuf = icon_pixbuf_stated;
+    }
+
+    return icon_pixbuf;
+}
