@@ -112,6 +112,20 @@ clientSameLeader (Client * c1, Client * c2)
              (c2->client_leader == c1->window)));
 }
 
+gboolean
+clientSameName (Client * c1, Client * c2)
+{
+    g_return_val_if_fail (c1 != NULL, FALSE);
+    g_return_val_if_fail (c2 != NULL, FALSE);
+
+    TRACE ("entering clientSameName");
+
+    return ((c1 != c2) &&
+            (c1->class.res_class != NULL) &&
+            (c2->class.res_class != NULL) &&
+            (strcmp (c1->class.res_name, c2->class.res_name) == 0));
+}
+
 gboolean
 clientIsTransientFor (Client * c1, Client * c2)
 {
@@ -255,10 +269,13 @@ clientTransientOrModalHasAncestor (Client * c, guint ws)
     for (list = screen_info->windows_stack; list; list = g_list_next (list))
     {
         c2 = (Client *) list->data;
-        if ((c2 != c) && !clientIsTransientOrModal (c2)
+        if ((c2 != c)
+            && !clientIsTransientOrModal (c2)
             && clientIsTransientOrModalFor (c, c2)
-            && !FLAG_TEST (c2->flags, CLIENT_FLAG_ICONIFIED)
-            && (c2->win_workspace == ws))
+            && FLAG_TEST (c2->xfwm_flags, XFWM_FLAG_VISIBLE)
+            && (c2->win_workspace == ws)
+            && (((ws == screen_info->current_ws) && FLAG_TEST (c2->xfwm_flags, XFWM_FLAG_VISIBLE))
+                || !FLAG_TEST (c2->flags, CLIENT_FLAG_ICONIFIED)))
         {
             return TRUE;
         }
@@ -442,7 +459,8 @@ clientSameApplication (Client *c1, Client *c2)
             clientIsTransientOrModalFor (c2, c1) ||
             clientSameGroup (c1, c2) ||
             clientSameLeader (c1, c2) ||
+            clientSameName (c1, c2) ||
             (c1->pid != 0 && c1->pid == c2->pid &&
              c1->hostname && c2->hostname &&
-             !g_strcasecmp (c1->hostname, c2->hostname)));
+             !g_ascii_strcasecmp (c1->hostname, c2->hostname)));
 }
